@@ -56,6 +56,7 @@ struct UICStruct {
 	gboolean getCalls;
 	gboolean getSms;
 	gulong sigId;
+	gboolean phoneOpened;
 };
 typedef struct UICStruct UIComponents;
 
@@ -92,6 +93,11 @@ static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *me
 	gconf_client_suggest_sync (gconf_client, NULL);
 	g_object_unref (gconf_client);
 
+	if (uic->phoneOpened) {
+		uic->phoneOpened = !(uic->phoneOpened);
+		gn_lib_phone_close(state);
+	}
+		
         gn_lib_phoneprofile_free(&state);
         gn_lib_library_free();
 
@@ -268,15 +274,30 @@ void on_menu_getCalls(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
 	
 	uic->getCalls = !(uic->getCalls);
 	if (uic->getCalls) {
-		/* Set up so that we get notified of incoming calls, if supported by the driver */
-		gn_data_clear(&data);
-		data.call_notification = notify_call;
-		data.callback_data = uic;
-		error = gn_sm_functions(GN_OP_SetCallNotification, &data, state);
+		if (!uic->phoneOpened) {
+			error = gn_lib_phone_open(state);
+		}
+		if (error == GN_ERR_NONE) {
+			/* Set up so that we get notified of incoming calls, if supported by the driver */
+			uic->phoneOpened = TRUE;
+			gn_data_clear(&data);
+			data.call_notification = notify_call;
+			data.callback_data = uic;
+			error = gn_sm_functions(GN_OP_SetCallNotification, &data, state);
+		}
 		if (error != GN_ERR_NONE) {
 			hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(GTK_WIDGET(checkMenuitem))), GTK_STOCK_DIALOG_INFO, _("This driver doesn't support call signaling.\n(%s)"), gn_error_print(error));
 			uic->getCalls = !(uic->getCalls);
 			gtk_check_menu_item_set_active(checkMenuitem, uic->getCalls);
+			if (uic->phoneOpened) {
+				uic->phoneOpened = !(uic->phoneOpened);
+				gn_lib_phone_close(state);
+			}
+		}
+	} else {
+		if (uic->phoneOpened) {
+			uic->phoneOpened = !(uic->phoneOpened);
+			gn_lib_phone_close(state);
 		}
 	}
 }
@@ -532,7 +553,7 @@ static guint dialSip(gchar * number, gchar ** message, UIComponents *uic) {
 	return 0;
 }
 
-static guint dialGsm(gchar *number, gchar **message) {
+static guint dialGsm(gchar *number, gchar **message, gboolean phoneOpened) {
 
 	guint ret = 0;
 
@@ -542,7 +563,7 @@ static guint dialGsm(gchar *number, gchar **message) {
         int call_id;
 				
 	/* initialise the connection */
-	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
+	if (!phoneOpened && (error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
 		g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
 		return 1;
 	}
@@ -560,7 +581,9 @@ static guint dialGsm(gchar *number, gchar **message) {
 		g_print("Dialled call, id: %d (lowlevel id: %d)\n", call_id, call_info.call_id);
 	}
 	 /* tear down the connection */
-	gn_lib_phone_close(state);
+	if (!phoneOpened) {
+		gn_lib_phone_close(state);
+	}
 	return ret;
 																			 
 }
@@ -593,7 +616,7 @@ void dialNumber(UIComponents * uic, gchar *number) {
 		text=NULL;
 		guint ret = 0;
 		if (uic->gsm == TRUE) {
-			ret = dialGsm(normalizedNumber, &text);
+			ret = dialGsm(normalizedNumber, &text, uic->phoneOpened);
 		} else {
 			sipNormalizedNumber = normalizeNumber4Sip(normalizedNumber, uic->transformPlus);
 			ret = dialSip(sipNormalizedNumber, &text, uic);
@@ -776,7 +799,7 @@ static void sendSMS_cb (GtkButton *widget, UIComponents *uic) {
 		if (text) {
 			showMessageSendSMS(GTK_WIDGET(widget), normalizedNumber);
 			/* initialise the connection */
-			if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
+			if (!uic->phoneOpened && (error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
 				g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
 				return;
 			}
@@ -785,7 +808,9 @@ static void sendSMS_cb (GtkButton *widget, UIComponents *uic) {
 			g_free(text);
 
 			/* tear down the connection */
-			gn_lib_phone_close(state);
+			if (!uic->phoneOpened) {
+				gn_lib_phone_close(state);
+			}
 		}
 		g_free(normalizedNumber);
 	}
@@ -877,7 +902,7 @@ static void sendSMSToTree (UIComponents *uic) {
 	gn_error error;
 	
 	/* initialise the connection */
-	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
+	if (!uic->phoneOpened && (error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
 		g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
 		return;
 	}
@@ -885,7 +910,9 @@ static void sendSMSToTree (UIComponents *uic) {
 	gtk_tree_model_foreach (GTK_TREE_MODEL(uic->storeNumber), (GtkTreeModelForeachFunc)sendSMSToIter, (gpointer)uic);
 
 	/* tear down the connection */
-	gn_lib_phone_close(state);
+	if (!uic->phoneOpened) {
+		gn_lib_phone_close(state);
+	}
 }
 
 gboolean clearSMSTextView_cb(GtkTextView *smsTextView, GdkEventFocus *event, UIComponents *uic) {
